/*********************************************************************
Project	:	GUSI				-	Grand Unified Socket Interface
File		:	GUSI.h			-	Socket calls
Author	:	Matthias Neeracher
Started	:	21Mar92								Language	:	MPW C/C++
Modified	:	17Apr92	MN	bzero()
				19Apr92	MN	C++ compatibility
				27Apr92	MN	getsockopt()
				18May92	MN	PPC stuff
				26Jun92	MN	choose()
				21Jul92	MN	sockaddr_atlk_sym
				03Aug92	MN	sa_constr_ppc
				07Sep92	MN	readlink()
				12Sep92	MN	Hostname stuff
				26Sep92	MN	Separate dirent and stat
				15Nov92	MN	remove netdb.h definitions
				08Dec92	MN	getcwd()
				14Feb93	MN	AF_PAP
				20Jun93	MN	Changed sa_constr_ppc
				27Jun93	MN	f?truncate
Last		:	27Jun93
*********************************************************************/

#ifndef _GUSI_
#define _GUSI_

#include <sys/types.h>

/* Feel free to increase FD_SETSIZE as needed */
#define GUSI_MAX_FD	FD_SETSIZE

#include <sys/cdefs.h>
#include <compat.h>
#include <sys/ioctl.h>
#include <sys/fcntl.h>
#include <sys/stat.h>
#include <dirent.h>
#include <Types.h>
#include <Events.h>
#include <Files.h>
#include <AppleTalk.h>
#include <CTBUtilities.h>
#include <Packages.h>
#include <PPCToolBox.h>
#include <sys/time.h>
#include <sys/socket.h>
#include <string.h>
#include <netinet/in.h>
#include <netdb.h>
#include <sys/un.h>
#include <unistd.h>
#include <machine/endian.h>

typedef enum spin_msg {
	SP_MISC,				/* some weird thing, usually just return immediately if you get this */
	SP_SELECT,			/* in a select call */
	SP_NAME,				/* getting a host by name */
	SP_ADDR,				/* getting a host by address */
	SP_STREAM_READ,	/* Stream read call */
	SP_STREAM_WRITE,	/* Stream write call */
	SP_DGRAM_READ,		/* Datagram read call */
	SP_DGRAM_WRITE,	/* Datagram write call */
	SP_SLEEP,			/* sleeping, passes ticks left to sleep */
	SP_AUTO_SPIN		/* Autospin, passes argument to SpinCursor */
} spin_msg;

typedef int (*GUSISpinFn)(spin_msg msg, long param);
typedef void (*GUSIEvtHandler)(EventRecord * ev);
typedef GUSIEvtHandler	GUSIEvtTable[24];

/*
 * Address families, defined in sys/socket.h
 *
 
#define	AF_UNSPEC		 0		// unspecified
#define	AF_UNIX			 1		// local to host (pipes, portals)
#define	AF_INET			 2		// internetwork: UDP, TCP, etc.
#define	AF_CTB			 3		// Apple Comm Toolbox (not yet supported)
#define	AF_FILE			 4		// Normal File I/O (used internally)
#define	AF_PPC			 5		// PPC Toolbox
#define	AF_PAP			 6		// Printer Access Protocol (client only)
#define	AF_REMOTECON	 7		// Remote Console (used internally)
#define	AF_APPLETALK	16		// Apple Talk

*/

#define	ATALK_SYMADDR 272		/* Symbolic Address for AppleTalk 			*/

/*
 * Types,  defined in sys/socket.h
 *

#define	SOCK_STREAM		 1		// stream socket 
#define	SOCK_DGRAM		 2		// datagram socket

*/

/*
 * Defined in sys/un.h
 *
 
struct sockaddr_un {
	short		sun_family;
	char 		sun_path[108];
};

*/

struct sockaddr_atlk {
	short			family;
	AddrBlock	addr;
};

struct sockaddr_atlk_sym {
	short			family;
	EntityName	name;
};

struct sockaddr_ppc {
	short					family;
	LocationNameRec	location;
	PPCPortRec			port;
};

/* Definitions for choose() */

#define 	CHOOSE_DEFAULT	1		/*	Use *name as default name						*/
#define	CHOOSE_NEW		2		/* Choose new entity name, not existing one	*/
#define	CHOOSE_DIR		4		/* Choose a directory name, not a file 		*/

typedef struct {
	short			numTypes;
	SFTypeList	types;
} sa_constr_file;

typedef struct {
	short			numTypes;
	NLType		types;
} sa_constr_atlk;

/* Definitions for sa_constr_ppc */

#define PPC_CON_NEWSTYLE		0x8000	/* Required */
#define PPC_CON_MATCH_NAME		0x0001	/* Match name */
#define PPC_CON_MATCH_TYPE 	0x0002 	/* Match port type */
#define PPC_CON_MATCH_NBP		0x0004	/* Match NBP type */

typedef struct	{
	short			flags;
	Str32			nbpType;
	PPCPortRec	match;
} sa_constr_ppc;

__BEGIN_DECLS
/* 
 * IO/Socket stuff, defined elsewhere (unistd.h, sys/socket.h
 *

int socket(int domain, int type, short protocol);
int bind(int s, void *name, int namelen);
int connect(int s, void *addr, int addrlen);
int listen(int s, int qlen);
int accept(int s, void *addr, int *addrlen);
int close(int s);
int read(int s, char *buffer, unsigned buflen);
int readv(int s, struct iovec *iov, int count);
int recv(int s, void *buffer, int buflen, int flags);
int recvfrom(int s, void *buffer, int buflen, int flags, void *from, int *fromlen);
int recvmsg(int s,struct msghdr *msg,int flags);
int write(int s, const char *buffer, unsigned buflen);
int writev(int s, struct iovec *iov, int count);
int send(int s, void *buffer, int buflen, int flags);
int sendto (int s, void *buffer, int buflen, int flags, void *to, int tolen);
int sendmsg(int s,struct msghdr *msg,int flags);
int select(int width, fd_set *readfds, fd_set *writefds, fd_set *exceptfds, struct timeval *timeout);
int getdtablesize(void);
int getsockname(int s, void *name, int *namelen);
int getpeername(int s, struct sockaddr *name, int *namelen);
int shutdown(int s, int how);
int fcntl(int s, unsigned int cmd, int arg);
int dup(int s);
int dup2(int s, int s1);
int ioctl(int d, unsigned int request, long *argp);
int getsockopt(int s, int level, int optname, char *optval, int * optlen);
int setsockopt(int s, int level, int optname, char *optval, int optlen);
int isatty(int);
int remove(const char *filename);
int rename(const char *oldname, const char *newname);
int creat(const char*);
int faccess(char*, unsigned int, long*);
long lseek(int, long, int);
int open(const char*, int);
int unlink(char*);
int symlink(char* linkto, char* linkname);
int readlink(char* path, char* buf, int bufsiz);
int truncate(char *path, long length);
int ftruncate(int fd, long length);
int chdir(char * path);
int mkdir(char * path);
int rmdir(char * path);
char * getcwd(char * buf, int size);
*/

/* 
 * Defined in stdio.h
 *
 
void fsetfileinfo (char *filename, unsigned long newcreator, unsigned long newtype);

*/

void fgetfileinfo (char *filename, unsigned long * creator, unsigned long * type);

int choose(
		int 		domain,
		int 		type,
		char * 	prompt,
		void * 	constraint,
		int 		flags,
		void * 	name,
		int * 	namelen);

/* 
 * Hostname routines, defined in netdb.h
 *
 
struct hostent * gethostbyname(char *name);
struct hostent * gethostbyaddr(struct in_addr *addrP, int, int);
int gethostname(char *machname, long buflen);
struct servent * getservbyname (char * name, char * proto);
struct protoent * getprotobyname(char * name);

*/

char * inet_ntoa(struct in_addr inaddr);
struct in_addr inet_addr(char *address);

/* 
 * What to do when a routine blocks
 */

void GUSISetSpin(GUSISpinFn routine);
GUSISpinFn GUSIGetSpin(void);
int GUSISetEvents(GUSIEvtTable table);
GUSIEvtHandler * GUSIGetEvents(void);

extern GUSIEvtHandler	GUSISIOWEvents[];

/* Directory Stuff */

/* 
 * BSD memory routines, defined in compat.h
 *

#define index(a, b)						strchr(a, b)
#define rindex(a, b)						strrchr(a, b)
#define bzero(from, len) 				memset(from, 0, len)
#define bcopy(from, to, len)			memcpy(to, from, len)
#define bcmp(s1, s2, len)				memcmp(s1, s2, len)
#define bfill(from, len, x)			memset(from, x, len)

 */

__END_DECLS

#endif /* !_GUSI_ */